@@ -1,60 +1,58 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
-#include <qtest.h>
-#include <QSignalSpy>
-#include <QTimer>
-#include <QHostAddress>
-#include <QDebug>
-#include <QThread>
-#include <QModelIndex>
 
-#include <QtQml/qqmlengine.h>
-#include <QtQml/qqmlcontext.h>
-#include <QtQml/qqmlcomponent.h>
-#include <QtQml/qqmlexpression.h>
-#include <QtQml/qqmlproperty.h>
-#include <QtQuick/qquickitem.h>
+#include "qqmlenginedebugclient.h"
+#include "debugutil_p.h"
+#include "../../../shared/util.h"
 
 #include <private/qqmlbinding_p.h>
 #include <private/qqmlboundsignal_p.h>
 #include <private/qqmldebugservice_p.h>
 #include <private/qqmlmetatype_p.h>
 #include <private/qqmlproperty_p.h>
+#include <private/qqmldebugconnection_p.h>
 
-#include "debugutil_p.h"
-#include "qqmlenginedebugclient.h"
+#include <QtTest/qtest.h>
+#include <QtTest/qsignalspy.h>
 
-#include "../../../shared/util.h"
+#include <QtQml/qqmlengine.h>
+#include <QtQml/qqmlcontext.h>
+#include <QtQml/qqmlcomponent.h>
+#include <QtQml/qqmlexpression.h>
+#include <QtQml/qqmlproperty.h>
+#include <QtQml/qqmlincubator.h>
+#include <QtQuick/qquickitem.h>
+
+#include <QtNetwork/qhostaddress.h>
+#include <QtCore/qtimer.h>
+#include <QtCore/qdebug.h>
+#include <QtCore/qthread.h>
+#include <QtCore/qabstractitemmodel.h>
 
 #define QVERIFYOBJECT(statement) \
     do {\
@@ -87,6 +85,8 @@ public:
 class tst_QQmlEngineDebugService : public QObject
 {
     Q_OBJECT
+public:
+    tst_QQmlEngineDebugService() : m_conn(0), m_dbg(0), m_engine(0), m_rootItem(0) {}
 
 private:
     QmlDebugObjectReference findRootObject(int context = 0,
@@ -137,6 +137,7 @@ private slots:
 
     void regression_QTCREATORBUG_7451();
     void queryObjectWithNonStreamableTypes();
+    void asynchronousCreate();
 };
 
 QmlDebugObjectReference tst_QQmlEngineDebugService::findRootObject(
@@ -179,14 +180,9 @@ void tst_QQmlEngineDebugService::recursiveObjectTest(
 {
     const QMetaObject *meta = o->metaObject();
 
-    QQmlType *type = QQmlMetaType::qmlType(meta);
-    QString className = type ? QString(type->qmlTypeName())
-                             : QString(meta->className());
-    className = className.mid(className.lastIndexOf(QLatin1Char('/'))+1);
-
     QCOMPARE(oref.debugId, QQmlDebugService::idForObject(o));
     QCOMPARE(oref.name, o->objectName());
-    QCOMPARE(oref.className, className);
+    QCOMPARE(oref.className, QQmlMetaType::prettyTypeName(o));
     QCOMPARE(oref.contextDebugId, QQmlDebugService::idForObject(
                  qmlContext(o)));
 
@@ -200,6 +196,7 @@ void tst_QQmlEngineDebugService::recursiveObjectTest(
 
         QmlDebugObjectReference cref;
         foreach (const QmlDebugObjectReference &ref, oref.children) {
+            QVERIFY(!ref.className.isEmpty());
             if (ref.debugId == debugId) {
                 cref = ref;
                 break;
@@ -349,9 +346,12 @@ void tst_QQmlEngineDebugService::initTestCase()
 
     bool ok = m_conn->waitForConnected();
     QVERIFY(ok);
-    QTRY_VERIFY(QQmlDebugService::hasDebuggingClient());
     m_dbg = new QQmlEngineDebugClient(m_conn);
-    QTRY_VERIFY(m_dbg->state() == QQmlEngineDebugClient::Enabled);
+    QList<QQmlDebugClient *> others = QQmlDebugTest::createOtherClients(m_conn);
+    QTRY_COMPARE(m_dbg->state(), QQmlEngineDebugClient::Enabled);
+    foreach (QQmlDebugClient *other, others)
+        QCOMPARE(other->state(), QQmlDebugClient::Unavailable);
+    qDeleteAll(others);
 }
 
 void tst_QQmlEngineDebugService::cleanupTestCase()
@@ -365,6 +365,7 @@ void tst_QQmlEngineDebugService::setMethodBody()
 {
     bool success;
     QmlDebugObjectReference obj = findRootObject(2);
+    QVERIFY(!obj.className.isEmpty());
 
     QObject *root = m_components.at(2);
     // Without args
@@ -372,7 +373,7 @@ void tst_QQmlEngineDebugService::setMethodBody()
     QVariant rv;
     QVERIFY(QMetaObject::invokeMethod(root, "myMethodNoArgs", Qt::DirectConnection,
                                       Q_RETURN_ARG(QVariant, rv)));
-    QVERIFY(rv == QVariant(qreal(3)));
+    QCOMPARE(rv, QVariant(qreal(3)));
 
 
     QVERIFY(m_dbg->setMethodBody(obj.debugId, "myMethodNoArgs", "return 7",
@@ -382,7 +383,7 @@ void tst_QQmlEngineDebugService::setMethodBody()
 
     QVERIFY(QMetaObject::invokeMethod(root, "myMethodNoArgs", Qt::DirectConnection,
                                       Q_RETURN_ARG(QVariant, rv)));
-    QVERIFY(rv == QVariant(qreal(7)));
+    QCOMPARE(rv, QVariant(qreal(7)));
     }
 
     // With args
@@ -390,7 +391,7 @@ void tst_QQmlEngineDebugService::setMethodBody()
     QVariant rv;
     QVERIFY(QMetaObject::invokeMethod(root, "myMethod", Qt::DirectConnection,
                                       Q_RETURN_ARG(QVariant, rv), Q_ARG(QVariant, QVariant(19))));
-    QVERIFY(rv == QVariant(qreal(28)));
+    QCOMPARE(rv, QVariant(qreal(28)));
 
     QVERIFY(m_dbg->setMethodBody(obj.debugId, "myMethod", "return a + 7",
                                  &success));
@@ -399,13 +400,14 @@ void tst_QQmlEngineDebugService::setMethodBody()
 
     QVERIFY(QMetaObject::invokeMethod(root, "myMethod", Qt::DirectConnection,
                                       Q_RETURN_ARG(QVariant, rv), Q_ARG(QVariant, QVariant(19))));
-    QVERIFY(rv == QVariant(qreal(26)));
+    QCOMPARE(rv, QVariant(qreal(26)));
     }
 }
 
 void tst_QQmlEngineDebugService::watch_property()
 {
     QmlDebugObjectReference obj = findRootObject();
+    QVERIFY(!obj.className.isEmpty());
     QmlDebugPropertyReference prop = findProperty(obj.properties, "width");
 
     bool success;
@@ -450,6 +452,7 @@ void tst_QQmlEngineDebugService::watch_property()
 void tst_QQmlEngineDebugService::watch_object()
 {
     QmlDebugObjectReference obj = findRootObject();
+    QVERIFY(!obj.className.isEmpty());
 
     bool success;
 
@@ -515,6 +518,7 @@ void tst_QQmlEngineDebugService::watch_expression()
     int origWidth = m_rootItem->property("width").toInt();
 
     QmlDebugObjectReference obj = findRootObject();
+    QVERIFY(!obj.className.isEmpty());
 
     bool success;
 
@@ -650,6 +654,7 @@ void tst_QQmlEngineDebugService::queryObject()
     bool success;
 
     QmlDebugObjectReference rootObject = findRootObject();
+    QVERIFY(!rootObject.className.isEmpty());
 
     QQmlEngineDebugClient *unconnected = new QQmlEngineDebugClient(0);
     recursive ? unconnected->queryObjectRecursive(rootObject, &success) : unconnected->queryObject(rootObject, &success);
@@ -661,6 +666,7 @@ void tst_QQmlEngineDebugService::queryObject()
     QVERIFY(QQmlDebugTest::waitForSignal(m_dbg, SIGNAL(result())));
 
     QmlDebugObjectReference obj = m_dbg->object();
+    QVERIFY(!obj.className.isEmpty());
 
     // check source as defined in main()
     QmlDebugFileReference source = obj.source;
@@ -672,12 +678,15 @@ void tst_QQmlEngineDebugService::queryObject()
     recursiveObjectTest(m_rootItem, obj, recursive);
 
     if (recursive) {
-        foreach (const QmlDebugObjectReference &child, obj.children)
+        foreach (const QmlDebugObjectReference &child, obj.children) {
+            QVERIFY(!child.className.isEmpty());
             QVERIFY(child.properties.count() > 0);
+        }
 
         QmlDebugObjectReference rect;
         QmlDebugObjectReference text;
         foreach (const QmlDebugObjectReference &child, obj.children) {
+            QVERIFY(!child.className.isEmpty());
             if (child.className == "Rectangle")
                 rect = child;
             else if (child.className == "Text")
@@ -691,8 +700,10 @@ void tst_QQmlEngineDebugService::queryObject()
 
         QCOMPARE(findProperty(text.properties, "color").value, qVariantFromValue(QColor("blue")));
     } else {
-        foreach (const QmlDebugObjectReference &child, obj.children)
+        foreach (const QmlDebugObjectReference &child, obj.children) {
+            QVERIFY(!child.className.isEmpty());
             QCOMPARE(child.properties.count(), 0);
+        }
     }
 }
 
@@ -711,6 +722,7 @@ void tst_QQmlEngineDebugService::queryObjectsForLocation()
     bool success;
 
     QmlDebugObjectReference rootObject = findRootObject();
+    QVERIFY(!rootObject.className.isEmpty());
 
     const QString fileName = QFileInfo(rootObject.source.url.toString()).fileName();
     int lineNumber = rootObject.source.lineNumber;
@@ -731,8 +743,9 @@ void tst_QQmlEngineDebugService::queryObjectsForLocation()
     QVERIFY(success);
     QVERIFY(QQmlDebugTest::waitForSignal(m_dbg, SIGNAL(result())));
 
-    QVERIFY(m_dbg->objects().count() == 1);
+    QCOMPARE(m_dbg->objects().count(), 1);
     QmlDebugObjectReference obj = m_dbg->objects().first();
+    QVERIFY(!obj.className.isEmpty());
 
     // check source as defined in main()
     QmlDebugFileReference source = obj.source;
@@ -744,12 +757,15 @@ void tst_QQmlEngineDebugService::queryObjectsForLocation()
     recursiveObjectTest(m_rootItem, obj, recursive);
 
     if (recursive) {
-        foreach (const QmlDebugObjectReference &child, obj.children)
+        foreach (const QmlDebugObjectReference &child, obj.children) {
+            QVERIFY(!child.className.isEmpty());
             QVERIFY(child.properties.count() > 0);
+        }
 
         QmlDebugObjectReference rect;
         QmlDebugObjectReference text;
         foreach (const QmlDebugObjectReference &child, obj.children) {
+            QVERIFY(!child.className.isEmpty());
             if (child.className == "Rectangle")
                 rect = child;
             else if (child.className == "Text")
@@ -763,8 +779,10 @@ void tst_QQmlEngineDebugService::queryObjectsForLocation()
 
         QCOMPARE(findProperty(text.properties, "color").value, qVariantFromValue(QColor("blue")));
     } else {
-        foreach (const QmlDebugObjectReference &child, obj.children)
+        foreach (const QmlDebugObjectReference &child, obj.children) {
+            QVERIFY(!child.className.isEmpty());
             QCOMPARE(child.properties.count(), 0);
+        }
     }
 }
 
@@ -779,6 +797,7 @@ void tst_QQmlEngineDebugService::queryObjectsForLocation_data()
 void tst_QQmlEngineDebugService::regression_QTCREATORBUG_7451()
 {
     QmlDebugObjectReference rootObject = findRootObject();
+    QVERIFY(!rootObject.className.isEmpty());
     int contextId = rootObject.contextDebugId;
     QQmlContext *context = qobject_cast<QQmlContext *>(QQmlDebugService::objectForId(contextId));
     QQmlComponent component(context->engine());
@@ -805,6 +824,7 @@ void tst_QQmlEngineDebugService::regression_QTCREATORBUG_7451()
     QVERIFY(QQmlDebugTest::waitForSignal(m_dbg, SIGNAL(result())));
 
     foreach (QmlDebugObjectReference child, rootObject.children) {
+        QVERIFY(!child.className.isEmpty());
         success = false;
         lineNumber = child.source.lineNumber;
         columnNumber = child.source.columnNumber;
@@ -827,6 +847,7 @@ void tst_QQmlEngineDebugService::regression_QTCREATORBUG_7451()
     QVERIFY(QQmlDebugTest::waitForSignal(m_dbg, SIGNAL(result())));
 
     foreach (QmlDebugObjectReference child, rootObject.children) {
+        QVERIFY(!child.className.isEmpty());
         success = false;
         lineNumber = child.source.lineNumber;
         columnNumber = child.source.columnNumber;
@@ -842,6 +863,7 @@ void tst_QQmlEngineDebugService::queryObjectWithNonStreamableTypes()
     bool success;
 
     QmlDebugObjectReference rootObject = findRootObject(4, true);
+    QVERIFY(!rootObject.className.isEmpty());
 
     QQmlEngineDebugClient *unconnected = new QQmlEngineDebugClient(0);
     unconnected->queryObject(rootObject, &success);
@@ -853,6 +875,7 @@ void tst_QQmlEngineDebugService::queryObjectWithNonStreamableTypes()
     QVERIFY(QQmlDebugTest::waitForSignal(m_dbg, SIGNAL(result())));
 
     QmlDebugObjectReference obj = m_dbg->object();
+    QVERIFY(!obj.className.isEmpty());
 
     QCOMPARE(findProperty(obj.properties, "modelIndex").value, QVariant());
 }
@@ -946,6 +969,7 @@ void tst_QQmlEngineDebugService::queryExpressionResultBC_data()
 void tst_QQmlEngineDebugService::setBindingForObject()
 {
     QmlDebugObjectReference rootObject = findRootObject();
+    QVERIFY(!rootObject.className.isEmpty());
     QVERIFY(rootObject.debugId != -1);
     QmlDebugPropertyReference widthPropertyRef = findProperty(rootObject.properties, "width");
 
@@ -963,6 +987,7 @@ void tst_QQmlEngineDebugService::setBindingForObject()
     QCOMPARE(m_dbg->valid(), true);
 
     rootObject = findRootObject();
+    QVERIFY(!rootObject.className.isEmpty());
     widthPropertyRef =  findProperty(rootObject.properties, "width");
 
     QCOMPARE(widthPropertyRef.value, QVariant(15));
@@ -978,6 +1003,7 @@ void tst_QQmlEngineDebugService::setBindingForObject()
     QCOMPARE(m_dbg->valid(), true);
 
     rootObject = findRootObject();
+    QVERIFY(!rootObject.className.isEmpty());
     widthPropertyRef =  findProperty(rootObject.properties, "width");
 
     QCOMPARE(widthPropertyRef.value, QVariant(20));
@@ -988,13 +1014,14 @@ void tst_QQmlEngineDebugService::setBindingForObject()
     // set handler
     //
     rootObject = findRootObject();
+    QVERIFY(!rootObject.className.isEmpty());
     QCOMPARE(rootObject.children.size(), 5); // Rectangle, Text, MouseArea, Component.onCompleted, NonScriptPropertyElement
     QmlDebugObjectReference mouseAreaObject = rootObject.children.at(2);
+    QVERIFY(!mouseAreaObject.className.isEmpty());
     m_dbg->queryObjectRecursive(mouseAreaObject, &success);
     QVERIFY(success);
     QVERIFY(QQmlDebugTest::waitForSignal(m_dbg, SIGNAL(result())));
     mouseAreaObject = m_dbg->object();
-
     QCOMPARE(mouseAreaObject.className, QString("MouseArea"));
 
     QmlDebugPropertyReference onEnteredRef = findProperty(mouseAreaObject.properties, "onEntered");
@@ -1011,19 +1038,23 @@ void tst_QQmlEngineDebugService::setBindingForObject()
     QCOMPARE(m_dbg->valid(), true);
 
     rootObject = findRootObject();
+    QVERIFY(!rootObject.className.isEmpty());
     mouseAreaObject = rootObject.children.at(2);
+    QVERIFY(!mouseAreaObject.className.isEmpty());
     m_dbg->queryObjectRecursive(mouseAreaObject, &success);
     QVERIFY(success);
     QVERIFY(QQmlDebugTest::waitForSignal(m_dbg, SIGNAL(result())));
     mouseAreaObject = m_dbg->object();
+    QVERIFY(!mouseAreaObject.className.isEmpty());
     onEnteredRef = findProperty(mouseAreaObject.properties, "onEntered");
     QCOMPARE(onEnteredRef.name, QString("onEntered"));
-    QCOMPARE(onEnteredRef.value,  QVariant("{console.log('hello, world') }"));
+    QCOMPARE(onEnteredRef.value, QVariant("function() { [code] }"));
 }
 
 void tst_QQmlEngineDebugService::resetBindingForObject()
 {
     QmlDebugObjectReference rootObject = findRootObject();
+    QVERIFY(!rootObject.className.isEmpty());
     QVERIFY(rootObject.debugId != -1);
     QmlDebugPropertyReference widthPropertyRef = findProperty(rootObject.properties, "width");
 
@@ -1044,6 +1075,7 @@ void tst_QQmlEngineDebugService::resetBindingForObject()
     QCOMPARE(m_dbg->valid(), true);
 
     rootObject = findRootObject();
+    QVERIFY(!rootObject.className.isEmpty());
     widthPropertyRef =  findProperty(rootObject.properties, "width");
 
     QCOMPARE(widthPropertyRef.value, QVariant(0));
@@ -1059,6 +1091,7 @@ void tst_QQmlEngineDebugService::resetBindingForObject()
     QCOMPARE(m_dbg->valid(), true);
 
     rootObject = findRootObject();
+    QVERIFY(!rootObject.className.isEmpty());
     QmlDebugPropertyReference boldPropertyRef =  findProperty(rootObject.properties, "font.bold");
 
     QCOMPARE(boldPropertyRef.value.toBool(), false);
@@ -1072,7 +1105,7 @@ void tst_QQmlEngineDebugService::setBindingInStates()
     const int sourceIndex = 3;
 
     QmlDebugObjectReference obj = findRootObject(sourceIndex);
-
+    QVERIFY(!obj.className.isEmpty());
     QVERIFY(obj.debugId != -1);
     QVERIFY(obj.children.count() >= 2);
     bool success;
@@ -1088,6 +1121,7 @@ void tst_QQmlEngineDebugService::setBindingInStates()
     QVERIFY(QQmlDebugTest::waitForSignal(m_dbg, SIGNAL(result())));
 
     obj = findRootObject(sourceIndex);
+    QVERIFY(!obj.className.isEmpty());
     QCOMPARE(findProperty(obj.properties,"width").value.toInt(),200);
 
 
@@ -1098,6 +1132,7 @@ void tst_QQmlEngineDebugService::setBindingInStates()
 
 
     obj = findRootObject(sourceIndex, true);
+    QVERIFY(!obj.className.isEmpty());
     QCOMPARE(findProperty(obj.properties,"width").value.toInt(),100);
 
 
@@ -1107,6 +1142,7 @@ void tst_QQmlEngineDebugService::setBindingInStates()
     QVERIFY(state.children.count() > 0);
 
     QmlDebugObjectReference propertyChange = state.children[0];
+    QVERIFY(!propertyChange.className.isEmpty());
     QVERIFY(propertyChange.debugId != -1);
 
     m_dbg->setBindingForObject(propertyChange.debugId, "width",QVariant(300),true,
@@ -1116,6 +1152,7 @@ void tst_QQmlEngineDebugService::setBindingInStates()
 
     // check properties changed in state
     obj = findRootObject(sourceIndex);
+    QVERIFY(!obj.className.isEmpty());
     QCOMPARE(findProperty(obj.properties,"width").value.toInt(),100);
 
 
@@ -1124,6 +1161,7 @@ void tst_QQmlEngineDebugService::setBindingInStates()
     QVERIFY(QQmlDebugTest::waitForSignal(m_dbg, SIGNAL(result())));
 
     obj = findRootObject(sourceIndex);
+    QVERIFY(!obj.className.isEmpty());
     QCOMPARE(findProperty(obj.properties,"width").value.toInt(),300);
 
     // check changing properties of base state from within a state
@@ -1137,6 +1175,7 @@ void tst_QQmlEngineDebugService::setBindingInStates()
     QVERIFY(QQmlDebugTest::waitForSignal(m_dbg, SIGNAL(result())));
 
     obj = findRootObject(sourceIndex);
+    QVERIFY(!obj.className.isEmpty());
     QCOMPARE(findProperty(obj.properties,"width").value.toInt(),300);
 
     m_dbg->queryExpressionResult(obj.debugId,QString("state=\"\""), &success);
@@ -1144,6 +1183,7 @@ void tst_QQmlEngineDebugService::setBindingInStates()
     QVERIFY(QQmlDebugTest::waitForSignal(m_dbg, SIGNAL(result())));
 
     obj = findRootObject(sourceIndex);
+    QVERIFY(!obj.className.isEmpty());
     QCOMPARE(findProperty(obj.properties,"width").value.toInt(), 400);
 
     //  reset binding while in a state
@@ -1152,6 +1192,7 @@ void tst_QQmlEngineDebugService::setBindingInStates()
     QVERIFY(QQmlDebugTest::waitForSignal(m_dbg, SIGNAL(result())));
 
     obj = findRootObject(sourceIndex);
+    QVERIFY(!obj.className.isEmpty());
     QCOMPARE(findProperty(obj.properties,"width").value.toInt(), 300);
 
     m_dbg->resetBindingForObject(propertyChange.debugId, "width", &success);
@@ -1160,6 +1201,7 @@ void tst_QQmlEngineDebugService::setBindingInStates()
     QCOMPARE(m_dbg->valid(), true);
 
     obj = findRootObject(sourceIndex);
+    QVERIFY(!obj.className.isEmpty());
     QCOMPARE(findProperty(obj.properties,"width").value.toInt(), 400);
 
     // re-add binding
@@ -1170,6 +1212,7 @@ void tst_QQmlEngineDebugService::setBindingInStates()
     QCOMPARE(m_dbg->valid(), true);
 
     obj = findRootObject(sourceIndex);
+    QVERIFY(!obj.className.isEmpty());
     QCOMPARE(findProperty(obj.properties,"width").value.toInt(), 300);
 }
 
@@ -1178,7 +1221,7 @@ void tst_QQmlEngineDebugService::queryObjectTree()
     const int sourceIndex = 3;
 
     QmlDebugObjectReference obj = findRootObject(sourceIndex, true);
-
+    QVERIFY(!obj.className.isEmpty());
     QVERIFY(obj.debugId != -1);
     QVERIFY(obj.children.count() >= 2);
 
@@ -1188,16 +1231,16 @@ void tst_QQmlEngineDebugService::queryObjectTree()
     QVERIFY(state.children.count() > 0);
 
     QmlDebugObjectReference propertyChange = state.children[0];
+    QVERIFY(!propertyChange.className.isEmpty());
     QVERIFY(propertyChange.debugId != -1);
 
     QmlDebugPropertyReference propertyChangeTarget = findProperty(propertyChange.properties,"target");
     QCOMPARE(propertyChangeTarget.objectDebugId, propertyChange.debugId);
 
     QmlDebugObjectReference targetReference = qvariant_cast<QmlDebugObjectReference>(propertyChangeTarget.value);
+    QVERIFY(!targetReference.className.isEmpty());
     QVERIFY(targetReference.debugId != -1);
 
-
-
     // check transition
     QmlDebugObjectReference transition = obj.children[0];
     QCOMPARE(transition.className, QString("Transition"));
@@ -1206,31 +1249,58 @@ void tst_QQmlEngineDebugService::queryObjectTree()
     QVERIFY(transition.children.count() > 0);
 
     QmlDebugObjectReference animation = transition.children[0];
+    QVERIFY(!animation.className.isEmpty());
     QVERIFY(animation.debugId != -1);
 
     QmlDebugPropertyReference animationTarget = findProperty(animation.properties,"target");
     QCOMPARE(animationTarget.objectDebugId, animation.debugId);
 
     targetReference = qvariant_cast<QmlDebugObjectReference>(animationTarget.value);
+    QVERIFY(!targetReference.className.isEmpty());
     QVERIFY(targetReference.debugId != -1);
 
     QCOMPARE(findProperty(animation.properties,"property").value.toString(), QString("width"));
     QCOMPARE(findProperty(animation.properties,"duration").value.toInt(), 100);
 }
 
+void tst_QQmlEngineDebugService::asynchronousCreate() {
+    QmlDebugObjectReference object;
+    auto connection = connect(m_dbg, &QQmlEngineDebugClient::newObject, this, [&](int objectId) {
+        object.debugId = objectId;
+    });
+
+    QByteArray asynchronousComponent = "import QtQuick 2.5\n"
+                                       "Rectangle { id: asyncRect }";
+    QQmlComponent component(m_engine);
+    component.setData(asynchronousComponent, QUrl::fromLocalFile(""));
+    QVERIFY(component.isReady());  // fails if bad syntax
+    QQmlIncubator incubator(QQmlIncubator::Asynchronous);
+    component.create(incubator);
+
+    QVERIFY(m_dbg->object().idString != QLatin1String("asyncRect"));
+
+    QTRY_VERIFY(object.debugId != -1);
+    disconnect(connection);
+
+    bool success = false;
+    m_dbg->queryObject(object, &success);
+    QVERIFY(success);
+
+    QTRY_COMPARE(m_dbg->object().idString, QLatin1String("asyncRect"));
+}
+
 int main(int argc, char *argv[])
 {
     int _argc = argc + 1;
-    char **_argv = new char*[_argc];
+    QScopedArrayPointer<char *>_argv(new char*[_argc]);
     for (int i = 0; i < argc; ++i)
         _argv[i] = argv[i];
-    char arg[] = "-qmljsdebugger=port:3768";
+    char arg[] = "-qmljsdebugger=port:3768,services:QmlDebugger";
     _argv[_argc - 1] = arg;
 
-    QGuiApplication app(_argc, _argv);
+    QGuiApplication app(_argc, _argv.data());
     tst_QQmlEngineDebugService tc;
-    return QTest::qExec(&tc, _argc, _argv);
-    delete _argv;
+    return QTest::qExec(&tc, _argc, _argv.data());
 }
 
 #include "tst_qqmlenginedebugservice.moc"
