@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -136,7 +131,7 @@ public:
 
 void MyQmlObject::v8function(QQmlV4Function *function)
 {
-    QV8Engine::getV4(function->engine())->currentContext()->throwError(QStringLiteral("Exception thrown from within QObject slot"));
+    function->v4engine()->throwError(QStringLiteral("Exception thrown from within QObject slot"));
 }
 
 static QJSValue script_api(QQmlEngine *engine, QJSEngine *scriptEngine)
@@ -250,9 +245,16 @@ public:
     MyWorkerObject *o;
 };
 
+MyWorkerObject::~MyWorkerObject()
+{
+    if (m_thread)
+        m_thread->wait();
+}
+
 void MyWorkerObject::doIt()
 {
-    new MyWorkerObjectThread(this);
+    Q_ASSERT(!m_thread);
+    m_thread = new MyWorkerObjectThread(this);
 }
 
 class MyDateClass : public QObject
@@ -300,6 +302,18 @@ public:
     {
         return v;
     }
+    Q_INVOKABLE QVector<int> integerVector(QVector<int> v) const
+    {
+        return v;
+    }
+    Q_INVOKABLE QVector<qreal> realVector(QVector<qreal> v) const
+    {
+        return v;
+    }
+    Q_INVOKABLE QVector<bool> boolVector(QVector<bool> v) const
+    {
+        return v;
+    }
 };
 
 static MyInheritedQmlObject *theSingletonObject = 0;
@@ -325,6 +339,38 @@ bool MyInheritedQmlObject::isItYouMyInheritedQmlObject(MyInheritedQmlObject *o)
     return o && o == theSingletonObject;
 }
 
+class TestTypeCppSingleton : public QObject
+{
+    Q_OBJECT
+    Q_PROPERTY(int foo READ foo)
+
+    Q_CLASSINFO("DefaultProperty", "foo")
+public:
+    int foo() { return 0; }
+    static TestTypeCppSingleton *instance() {
+        static TestTypeCppSingleton cppSingleton;
+        return &cppSingleton;
+    }
+private:
+    TestTypeCppSingleton(){}
+    ~TestTypeCppSingleton() {
+        // just to make sure it crashes on double delete
+        static int a = 0;
+        static int *ptr = &a;
+        *ptr = 1;
+        ptr = 0;
+    }
+};
+
+QObject *testTypeCppProvider(QQmlEngine *engine, QJSEngine *scriptEngine)
+{
+    Q_UNUSED(engine);
+    Q_UNUSED(scriptEngine);
+    TestTypeCppSingleton *o = TestTypeCppSingleton::instance();
+    QQmlEngine::setObjectOwnership(o, QQmlEngine::CppOwnership);
+    return o;
+}
+
 static QObject *create_singletonWithEnum(QQmlEngine *, QJSEngine *)
 {
     return new SingletonWithEnum;
@@ -391,6 +437,7 @@ void registerTypes()
     qmlRegisterType<MyQmlObject>("Qt.test", 1,0, "MyQmlObjectAlias");
     qmlRegisterType<MyQmlObject>("Qt.test", 1,0, "MyQmlObject");
     qmlRegisterSingletonType<MyInheritedQmlObject>("Test", 1, 0, "MyInheritedQmlObjectSingleton", inheritedQmlObject_provider);
+    qmlRegisterSingletonType<TestTypeCppSingleton>("Test", 1, 0, "TestTypeCppSingleton", testTypeCppProvider);
     qmlRegisterType<MyDeferredObject>("Qt.test", 1,0, "MyDeferredObject");
     qmlRegisterType<MyVeryDeferredObject>("Qt.test", 1,0, "MyVeryDeferredObject");
     qmlRegisterType<MyQmlContainer>("Qt.test", 1,0, "MyQmlContainer");
@@ -419,6 +466,7 @@ void registerTypes()
     qmlRegisterType<MyRevisionedSubclass>("Qt.test",1,0,"MyRevisionedSubclass");
     // MyRevisionedSubclass 1.1 uses MyRevisionedClass revision 1
     qmlRegisterType<MyRevisionedSubclass,1>("Qt.test",1,1,"MyRevisionedSubclass");
+    qmlRegisterType<MyItemUsingRevisionedObject>("Qt.test", 1, 0, "MyItemUsingRevisionedObject");
 
 #ifndef QT_NO_WIDGETS
     qmlRegisterExtendedType<QWidget,QWidgetDeclarativeUI>("Qt.test",1,0,"QWidget");
@@ -474,6 +522,8 @@ void registerTypes()
     qmlRegisterType<QObjectContainer>("Qt.test", 1, 0, "QObjectContainer");
     qmlRegisterType<QObjectContainerWithGCOnAppend>("Qt.test", 1, 0, "QObjectContainerWithGCOnAppend");
     qmlRegisterType<FloatingQObject>("Qt.test", 1, 0, "FloatingQObject");
+
+    qmlRegisterType<ClashingNames>("Qt.test", 1, 0, "ClashingNames");
 }
 
 #include "testtypes.moc"
